# -------------------------------------------------------------------
# Arizona Medpar data analysis with Mixture Link Poisson regression model
# Written with mixlink package version 0.1.5
# Andrew Raim
# Created 7/17/2017
# -------------------------------------------------------------------

library(mixlink)
library(Matrix)
library(coda)

set.seed(1234)

# ----- Load the data from the COUNT package -----
data(azpro, package = "COUNT")
head(azpro)
idx <- which(azpro$hospital >= 6.49 & azpro$hospital <= 6.51)
y <- azpro$los[idx]
X <- model.matrix(~ procedure + sex + admit + age75, data=azpro[idx,])
n <- nrow(X)
d <- ncol(X)

# ----- MCMC -----
J <- 8
Beta.init <- rep(0,d)
Pi.init <- normalize(1:J)
kappa.init <- 1

proposal <- list(var = as.matrix(bdiag(solve(t(X) %*% X), diag(0.5, J-1), 1.0)), scale = 0.2)
hyper <- list(var.Beta = 1000, alpha.Pi = rep(1, J), a.kappa = 1, b.kappa = 1/2)
metrop.out <- rwmetrop.mixlink.poisson(y, X, R = 55000, burn = 5000, thin = 20,
	invlink = exp, Beta.init = Beta.init, Pi.init = Pi.init,
	kappa.init = kappa.init, hyper = hyper, report.period = 1000,
	use.laplace.approx = TRUE, proposal = proposal, param.grp = NULL)

# Save draws
# Sorting the Pi draws sometimes helps to simple fix label switching
R.keep <- metrop.out$R.keep
Beta.mcmc <- mcmc(metrop.out$Beta.hist)
Pi.unordered.mcmc <- mcmc(metrop.out$Pi.hist)
Pi.mcmc <- mcmc( t(apply(metrop.out$Pi.hist, 1, sort)) )
kappa.mcmc <- mcmc(metrop.out$kappa.hist)

# ----- Check diagnostics -----
plot(Beta.mcmc)
plot(Pi.mcmc)
plot(kappa.mcmc)

acf(Beta.mcmc)
acf(Pi.mcmc)
acf(kappa.mcmc)

# ----- Print summary -----
print(metrop.out)

# ----- Compute DIC -----
DIC(metrop.out)

# ----- Compute predictions and PIs from posterior predictive distn -----
Y.new <- matrix(NA, R.keep, n)
for (r in 1:R.keep) {
	Beta <- Beta.mcmc[r,]
	Pi <- Pi.mcmc[r,]
	kappa <- kappa.mcmc[r]
	mean.hat <-  exp(X %*% Beta)
	Y.new[r,] <- r.mixlink.pois(n, mean.hat, Pi, kappa)
}
Y.summary <- data.frame(
	mean = apply(Y.new, 2, mean, na.rm = TRUE),
	lo = apply(Y.new, 2, quantile, prob = 0.025, na.rm = TRUE),
	hi = apply(Y.new, 2, quantile, prob = 0.975, na.rm = TRUE)
)

# ----- Plot predictions -----
# Histogram of y's for each setting of x
idx0000 <- which(X[,2] == 0 & X[,3] == 0 & X[,4] == 0 & X[,5] == 0)
idx0001 <- which(X[,2] == 0 & X[,3] == 0 & X[,4] == 0 & X[,5] == 1)
idx0010 <- which(X[,2] == 0 & X[,3] == 0 & X[,4] == 1 & X[,5] == 0)
idx0011 <- which(X[,2] == 0 & X[,3] == 0 & X[,4] == 1 & X[,5] == 1)
idx0100 <- which(X[,2] == 0 & X[,3] == 1 & X[,4] == 0 & X[,5] == 0)
idx0101 <- which(X[,2] == 0 & X[,3] == 1 & X[,4] == 0 & X[,5] == 1)
idx0110 <- which(X[,2] == 0 & X[,3] == 1 & X[,4] == 1 & X[,5] == 0)
idx0111 <- which(X[,2] == 0 & X[,3] == 1 & X[,4] == 1 & X[,5] == 1)
idx1000 <- which(X[,2] == 1 & X[,3] == 0 & X[,4] == 0 & X[,5] == 0)
idx1001 <- which(X[,2] == 1 & X[,3] == 0 & X[,4] == 0 & X[,5] == 1)
idx1010 <- which(X[,2] == 1 & X[,3] == 0 & X[,4] == 1 & X[,5] == 0)
idx1011 <- which(X[,2] == 1 & X[,3] == 0 & X[,4] == 1 & X[,5] == 1)
idx1100 <- which(X[,2] == 1 & X[,3] == 1 & X[,4] == 0 & X[,5] == 0)
idx1101 <- which(X[,2] == 1 & X[,3] == 1 & X[,4] == 0 & X[,5] == 1)
idx1110 <- which(X[,2] == 1 & X[,3] == 1 & X[,4] == 1 & X[,5] == 0)
idx1111 <- which(X[,2] == 1 & X[,3] == 1 & X[,4] == 1 & X[,5] == 1)

# Plot for first set of covariates
boxplot(y[idx0000], y[idx0001], y[idx0010], y[idx0011],
	y[idx0100], y[idx0101], y[idx0110], y[idx0111],
	xaxt='n', ylab = "y", ylim = c(0, 52))
axis(side = 1, at = 1:8, labels = c("0000", "0001", "0010", "0011",
	"0100", "0101", "0110", "0111"))
axis(side = 1, at = 1:8, padj = 1.5, labels = c(
	sprintf("%d obs", length(idx0000)),
	sprintf("%d obs", length(idx0001)),
	sprintf("%d obs", length(idx0010)),
	sprintf("%d obs", length(idx0011)),
	sprintf("%d obs", length(idx0100)),
	sprintf("%d obs", length(idx0101)),
	sprintf("%d obs", length(idx0110)),
	sprintf("%d obs", length(idx0111))
))

# Use one representative from each covariate setting to compute interval
# from posterior predictive distribution
lo <- c(Y.summary$lo[idx0000[1]], Y.summary$lo[idx0001[1]],
  Y.summary$lo[idx0010[1]], Y.summary$lo[idx0011[1]],
  Y.summary$lo[idx0100[1]], Y.summary$lo[idx0101[1]],
  Y.summary$lo[idx0110[1]], Y.summary$lo[idx0111[1]],
  Y.summary$lo[idx1000[1]], Y.summary$lo[idx1001[1]],
  Y.summary$lo[idx1010[1]], Y.summary$lo[idx1011[1]],
  Y.summary$lo[idx1100[1]], Y.summary$lo[idx1101[1]],
  Y.summary$lo[idx1110[1]], Y.summary$lo[idx1111[1]]
)
med <- c(Y.summary$mean[idx0000[1]], Y.summary$mean[idx0001[1]],
  Y.summary$mean[idx0010[1]], Y.summary$mean[idx0011[1]],
  Y.summary$mean[idx0100[1]], Y.summary$mean[idx0101[1]],
  Y.summary$mean[idx0110[1]], Y.summary$mean[idx0111[1]],
  Y.summary$mean[idx1000[1]], Y.summary$mean[idx1001[1]],
  Y.summary$mean[idx1010[1]], Y.summary$mean[idx1011[1]],
  Y.summary$mean[idx1100[1]], Y.summary$mean[idx1101[1]],
  Y.summary$mean[idx1110[1]], Y.summary$mean[idx1111[1]]
        
)
hi <- c(Y.summary$hi[idx0000[1]], Y.summary$hi[idx0001[1]],
  Y.summary$hi[idx0010[1]], Y.summary$hi[idx0011[1]],
  Y.summary$hi[idx0100[1]], Y.summary$hi[idx0101[1]],
  Y.summary$hi[idx0110[1]], Y.summary$hi[idx0111[1]],
  Y.summary$hi[idx1000[1]], Y.summary$hi[idx1001[1]],
  Y.summary$hi[idx1010[1]], Y.summary$hi[idx1011[1]],
  Y.summary$hi[idx1100[1]], Y.summary$hi[idx1101[1]],
  Y.summary$hi[idx1110[1]], Y.summary$hi[idx1111[1]]
)

eps <- 0.4
for (j in 1:8) {
	segments(x0 = j - eps, x1 = j + eps, y0 = lo[j], y1 = lo[j], lty = 1, col = "blue", lwd = 2)
	segments(x0 = j - eps, x1 = j + eps, y0 = hi[j], y1 = hi[j], lty = 1, col = "blue", lwd = 2)
}

# ----- Plot Randomized Quantile Residuals -----
rqres.pp <- residuals(metrop.out)
rqres <- colMeans(rqres.pp)

qqnorm(rqres); abline(c(0,1), lty = 2, col = "red", lwd = 2, main = "Normal Q-Q Plot")
plot(Y.summary$mean, rqres, xlab = "Predicted Count", ylab = "Residual", main = "Residuals vs. Predicted Values")
