# -------------------------------------------------------------------
# Hiroshima data analysis with Mixture Link Binomial regression model
# Written with mixlink package version 0.1.5
# Andrew Raim
# Created 7/17/2017
# The Hiroshima dataset can be obtained from the website for the book
# "Overdispersion Models in SAS" by Morel and Neerchal via
# http://www.sas.com.
# -------------------------------------------------------------------

library(mixlink)
library(coda)

set.seed(1234)

# ----- Prepare the data -----
dat <- read.table("hiroshima.dat", head = TRUE)

y <- dat$t
m <- dat$m
z1 <- scale(dat$t65d_gamma)
z2 <- scale(dat$t65d_neutron)

# A single dimension-reduced covariate
W <- cbind(dat$t65d_gamma, dat$t65d_neutron)
pca <- princomp(W, cor = TRUE)
PC1 <- pca$scores[,1]

X <- model.matrix(~ PC1 + I(PC1^2), data = dat)
n <- nrow(X)
d <- ncol(X)

# ----- MCMC -----
J <- 2
hyper <- list(
	VBeta = diag(1000, d),
	alpha = rep(1, J),
	kappa.a = 1,
	kappa.b = 0.5
)

proposal.VBeta <- solve(t(X) %*% X)
proposal.VPi <- diag(0.2^2, J-1)
proposal.Vkappa <- 0.25^2
proposal.Vpsi <- diag(0.5^2, J)

Beta.init <- rep(0,d)
Pi.init <- normalize(1:J)
kappa.init <- 1

# Pilot MCMC run, to find a decent proposal distribution
gibbs.out.pilot <- gibbs.mixlink(y, X, R = 15000, burn = 1000, thin = 10,
	report.period = 1000, Beta.init = Beta.init, Pi.init = Pi.init,
	kappa.init = kappa.init, proposal.VBeta = proposal.VBeta,
	proposal.VPi = proposal.VPi, proposal.Vkappa = proposal.Vkappa,
	proposal.Vpsi = proposal.Vpsi, hyper = hyper, trials = m,
	family = "binomial")

# MCMC
gibbs.out <- gibbs.mixlink(y, X, R = 55000, burn = 5000, thin = 50,
	report.period = 1000,
	Beta.init = as.numeric(tail(gibbs.out.pilot$Beta.hist, 1)),
	Pi.init = as.numeric(tail(gibbs.out.pilot$Pi.hist, 1)),
	kappa.init = as.numeric(tail(gibbs.out.pilot$kappa.hist, 1)),
	proposal.VBeta = 0.5*var(gibbs.out.pilot$Beta.hist),
	proposal.VPi = var((apply(gibbs.out.pilot$Pi.hist, 1, mlogit))),
	proposal.Vkappa = var(gibbs.out.pilot$kappa.hist),
	proposal.Vpsi = proposal.Vpsi,
	hyper = hyper, trials = m, family = "binomial")

# Save draws
R.keep <- gibbs.out$R.keep
Beta.mcmc <- mcmc(gibbs.out$Beta.hist)
kappa.mcmc <- mcmc(gibbs.out$kappa.hist)
Pi.mcmc <- mcmc( t(apply(gibbs.out$Pi.hist, 1, sort)) )		# Adjust Pi for label switching

# ----- Check diagnostics -----
plot(Beta.mcmc)
plot(Pi.mcmc)
plot(kappa.mcmc)

acf(Beta.mcmc)
acf(Pi.mcmc)
acf(kappa.mcmc)

# ----- Print summary -----
print(gibbs.out)

# ----- Compute DIC -----
DIC(gibbs.out)

# ----- Compute predictions and PIs from posterior predictive distn -----
Y.new <- matrix(NA, R.keep, n)
for (r in 1:R.keep) {
	Beta <- Beta.mcmc[r,]
	Pi <- Pi.mcmc[r,]
	kappa <- kappa.mcmc[r]
	mean.hat <-  plogis(X %*% Beta)
	Y.new[r,] <- r.mixlink.binom(n, mean.hat, Pi, kappa, m)
}
Y.summary <- data.frame(
	mean = apply(Y.new, 2, mean, na.rm = TRUE),
	lo = apply(Y.new, 2, quantile, prob = 0.025, na.rm = TRUE),
	hi = apply(Y.new, 2, quantile, prob = 0.975, na.rm = TRUE)
)

# ----- Plot predictions -----
plot(PC1, y/m, ylim = c(0,1), col = "gray30", xlab = "x")
points(PC1, Y.summary$mean / m, pch = 20, cex = 0.5, col = "blue")
points(PC1, Y.summary$lo / m, pch = 20, cex = 0.5, col = "brown")
points(PC1, Y.summary$hi / m, pch = 20, cex = 0.5, col = "brown")

# ----- Plot Randomized Quantile Residuals -----
rqres.pp <- residuals(gibbs.out)
rqres <- colMeans(rqres.pp)
qqnorm(rqres); abline(c(0,1), lty = 2, col = "red", lwd = 2, main = "Normal Q-Q Plot")
plot(Y.summary$mean / m, rqres, xlab = "Predicted Proportion", ylab = "Residual", main = "Residuals vs. Fitted Values")
